"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.createDriver = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _objectWithoutPropertiesLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutPropertiesLoose"));

var _isAbsoluteURL = _interopRequireDefault(require("axios/lib/helpers/isAbsoluteURL"));

var responseTypes = ['arraybuffer', 'blob', 'formData', 'json', 'text', null];

var getResponseData = function getResponseData(response, responseType) {
  if (responseTypes.indexOf(responseType) === -1) {
    throw new Error("responseType must be one of the following: 'arraybuffer', 'blob', 'formData', 'json', 'text', null");
  }

  if (responseType === null) {
    return Promise.resolve(null);
  }

  return response[responseType]();
};

var DummyAbortController = /*#__PURE__*/function () {
  function DummyAbortController() {}

  var _proto = DummyAbortController.prototype;

  /* eslint-disable-next-line class-methods-use-this */
  _proto.abort = function abort() {};

  return DummyAbortController;
}();

var createDriver = function createDriver(fetchInstance, _temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      _ref$baseURL = _ref.baseURL,
      baseURL = _ref$baseURL === void 0 ? '' : _ref$baseURL,
      _ref$AbortController = _ref.AbortController,
      AbortController = _ref$AbortController === void 0 ? DummyAbortController : _ref$AbortController;

  return function (_ref2) {
    var url = _ref2.url,
        _ref2$responseType = _ref2.responseType,
        responseType = _ref2$responseType === void 0 ? 'json' : _ref2$responseType,
        requestConfig = (0, _objectWithoutPropertiesLoose2["default"])(_ref2, ["url", "responseType"]);
    var abortSource = new AbortController();
    var responsePromise = fetchInstance((0, _isAbsoluteURL["default"])(url) ? url : baseURL + url, (0, _extends2["default"])({
      signal: abortSource.signal
    }, requestConfig)).then(function (response) {
      if (response.ok) {
        return getResponseData(response, responseType);
      }

      return response.json().then(function (data) {
        response.data = data;
        throw response;
      }, function () {
        throw response;
      });
    }).then(function (data) {
      return {
        data: data
      };
    });

    responsePromise.cancel = function () {
      return abortSource.abort();
    };

    return responsePromise;
  };
};

exports.createDriver = createDriver;